program define rireg, sortpreserve eclass
version 8.2
set more off

/* 	This .ado file can be used to implement Repeated-Imputation Inference (RII) 
	estimation in Stata.
	
	The basic syntax for the .ado file is as follows:
	
	rireg depvar [varlist] [weight] [if exp], command(string) [options(string) imp(string)]
	
	The command displays the RII estimates along with their adjusted stadard errors, 
	t-statistics in a standard STATA output table. The RII estimated coefficient
	vector and estimated variance covariance matrix are saved in Stata's system area and
	can be obtained by reference to e(b) or e(V).  The command assumes that the dataset contains
	a variable called "imp" ranging from 1 to 5 that identifies the implicates.
	
	WARNING: Note that the p-values displayed in the output table are computed using a normal distribution
	as opposed to the t-distribution with the appropriate degress of freedom.  This is because with
	RII estimate the degrees of freedom is different for each estimated coefficient - see Montalto and Sung (1996).
	The correct p-values are listed in an output matrix that is stored as e(out).
	I have rigged the code so that a warning message is displayed if the difference between
	the displayed and the correct p-values exceeds 0.005

	Usage Examples:
	1. To run an OLS regression of y1 on x1 and x2 you would type:
	mireg y1 x1 x2, command(reg)
	2. To omit the constant from this regression you would type:
	mireg y1 x1 x2, command(reg) options(noconstant)
	3. To request robust standard errors and to omit the constatn you would type:
	mireg y1 x1 x2, command(reg) options(noconstant robust)
	4. To run an Probit regression of z1 on x1 and x2 you would type:
	mireg z1 x1 x2, command(probit)

	Background:
	This .ado file is based on the discussion in Appendix A
	of Montalto and Sung (1996) and closely follows their notation.  Their paper
	is available at http://www.hec.ohio-state.edu/hanna/imput.htm .  Apparently,
	the classical reference on RII is Rubin (1987). As discussed in Montalto and Sung,
	the RII point estimate is simply the average of the point estimates across all
	implicates. The estimated variance of the the RII estimate equals the average of
	the variance estimates from each implicates plus an adjustment for the "between"
	imputation variance.  This last term adjusts the estimated variance to take account
	of nonresponse and imputation.  Also, as discussed in Montalto and Sung the
	appropriate number degree of freedom for inference is now a function of the
	increase in variance due to nonresponse.  Specifically, larger increases in
	variance due to non-repsonse are associated with fewer degrees of freedoma
	and reduced p-values.
	
	Sam Hanson
	03/24/05

*/

syntax varlist(min=2) [aweight fweight iweight pweight] [if], COMmand(string) [OPTions(string) IMP(string)]

/* Drop all scalars and matrices in working memory */
scalar drop _all
matrix drop _all

local varcount : word count `varlist'
local depvar : word 1 of `varlist'
local indvarlist : subinstr local varlist "`depvar'" "", word

/* Make sample: for Heckman we explicitly want to include cases where depvar is missing */
if "`command'" == "heckman" | "`command'" == "intreg" {
	tempvar touse
	mark `touse' `if'
	markout `touse' `indvarlist'
}
else {
        marksample touse
}

/* Define imp if not specificied by user */
if substr("`imp'",1,1) == "" {
	local imp "imp"
}

/* Count number of observations included in the regression */
/* Note the number of included households actually varies across the implicates */
/* The reported number is the average across the 5 implicates rounded to the nearest integer */
quietly count if `touse'==1
local obs = round(r(N)/5)

/* Preserve Data */
preserve

forvalues i = 1/5 {

	/* Estimate regression seperately for each implicate */
	quietly `command' `varlist' if `imp'==`i' & `touse'==1 [`weight' `exp'], `options'

	if "`command'" == "dprobit" {
		/* Save vector of marginal effects for each implicate */
		matrix b`i'=e(dfdx)
		/* Save standard errors of marginal effects for each implicate */
		matrix U`i'=e(se_dfdx)
		/* Create diagonal matrix with variance of marginal effects for each implicate */
		/* Note: we do not save the off-diagonal elements of U for dprobit */
		/* that is cov(bi,bj) for i ne j*/
		matrix U`i' = diag(vecdiag(U`i''*U`i'))

	}
	else {
		/* Save coefficient vector for each implicate */
		matrix b`i'=e(b)
		/* Save covariance matrix for each implicate */
		matrix U`i'=e(V)
	}

	/* Tabulate the R-Squared for each regression */
	/* For models, estimated using MLE this will be a psuedo-R2 */
	/* which is saved under e(r2_p) */
	capture confirm scalar e(r2_p)
	if _rc == 0 {
             scalar r2_`i' = e(r2_p)
	}
        capture confirm scalar e(r2)
	if _rc == 0 {
             scalar r2_`i' = e(r2)
	}

	/* Some commands (intreg) do not return any measure of goodness of fit */
	capture confirm scalar r2_`i'
        if _rc != 0 {
             local goodness = "none"
	}
	else {
             local goodness = "some"
	}

	/* Test for convergence of MLE */
	/* If V is still not full rank as of the last iterations, I assume that we have a problem */
        capture confirm scalar e(rank)
	if _rc == 0 {
             	local rank = e(rank)
             	matrix cols = colsof(e(V))
             	local cols = cols[1,1]
             	if `rank' < `cols' {
           		local convergence = "Convergence Not Achieved"
            		continue, break
		}
		else if `rank' == `cols' {
            		local convergence = "Achieved"
            	}

        }
	else {
       		local convergence = "Achieved"
       	}

}

if "`convergence'" == "Convergence Not Achieved" {

	ereturn clear
	ereturn local converge "`convergence'"
        di in red "WARNING: Convergence Not Achieved"

}
else {

	/* CALCULATE COEFFICIENT VECTOR */
	matrix b=(b1+b2+b3+b4+b5)/5

        if "`command'" == "dprobit" {
	   matrix dfdx = b
	}

	/* CALCULATE AVERAGE "WITHIN" IMPUTATION COVARIANCE MATRIX */
	matrix U = (U1+U2+U3+U4+U5)/5

	/* CALCULATE "BETWEEN" IMPUTATION COVARIANCE MATRIX  - JUST SAMPLE COVARIANCE MATRIX FOR b1,b2,b3,b4, & b5*/
	matrix B = (1/(5-1))*( (b1-b)'*(b1-b) + (b2-b)'*(b2-b) + (b3-b)'*(b3-b) + (b4-b)'*(b4-b) + (b5-b)'*(b5-b) )

	/* CALCULATE TOTAL COVARIANCE MATRIX OF POINT ESTIMATES */
	matrix V = U + (1 + (1/5))*B

	if "`goodness'" == "some" {
		/* CALCULATE AVERAGE R-SQUARED */
		scalar r2=(r2_1+r2_2+r2_3+r2_4+r2_5)/5
	}

	/* CALCULATE STANDARD ERRORS, T-STATISTICS, DEGREES OF FREEDOM &  P-VALUES FOR POINT ESTIMATES */

	local N = colsof(b)
	matrix t = J(1,`N',0)
	matrix r = J(1,`N',0)
	matrix df = J(1,`N',0)
	matrix p = J(1,`N',0)
	matrix out = J(`N',5,0)

	matrix S = vecdiag(V)

	local pwarn = 0

	forvalues k = 1/`N' {

		/* CALCULATE STANDARD ERRORS OF POINT ESTIMATES */
		matrix S[1,`k'] = sqrt(S[1,`k'])
		/* T-STATISTICS FOR POINT ESTIMATES AGAINT NULL THAT b[1,k] = 0 */
		matrix t[1,`k'] =  b[1,`k']/S[1,`k']
		/* COMPUTE %INCREASE IN TOTAL VARINACE UDE TO NON-RESPONSE("BETWEEN" IMPUTATION VARIANCE)*/
		/* Note that: r[1,`k'] = V[`k',`k']/U[`k',`k'] - 1 */
		matrix r[1,`k'] = (1 + (1/5))*(B[`k',`k']/U[`k',`k'])
		/* COMPUTE DEGREES OF FREEDOM FOR POINT ESTIMATES */
		matrix df[1,`k'] = (5-1)*( 1 + (1/r[1,`k']) )^2
		/* COMPUTE P VALUES FOR TWO SIDED TEST THAT b[1,k] = 0 */
		matrix p[1,`k'] = 2*ttail(df[1,`k'],abs(t[1,`k']))

		/* TEST TO SEE IF DISPLAYED P-VALUE DIFFER FROM TRUE P-VALUES */
		local pz = 2*(1 - norm(abs(t[1,`k'])))
		if abs(p[1,`k'] - `pz') > 0.005 {
			local pwarn = `pwarn' + 1
		}

		/* Fill in output matrix */
		matrix out[`k',1] = b[1,`k']
		matrix out[`k',2] = S[1,`k']
		matrix out[`k',3] = t[1,`k']
		matrix out[`k',4] = df[1,`k']
		matrix out[`k',5] = p[1,`k']

	}

	/* DISPLAY WARNING */
	if `pwarn' > 0 {
		di in red "WARNING: Display p-values may differ significantly from RII p-values"
	}

	/* LABEL ROWS AND COLUMNS OF OUTPUT*/
	matrix colnames out = b std t df p
	local names: colfullnames b
	matrix rownames out = `names'

	/* CALCULATE F-STATISTIC FOR JOINT SIGNIFICANCE OF ALL COEFFICIENTS */

	matrix rF =  (1+(1/5))*(trace(B*syminv(U)))/`N'
	local rF = rF[1,1]
	local vF = (5-1)*( 1 + (1/`rF'))^2
	local n1 = `N'
	local n2 = (`N' + 1)*(`vF'/2)
	matrix F = (1/(1+`rF'))*(b*syminv(U)*b')/`N'
	local F = F[1,1]
	local pF = Ftail(`n1',`n2',`F')

	/* Post results */
	ereturn post b V, depname("`depvar'") obs(`obs')

	if "`command'" == "dprobit" {
	   ereturn matrix dfdx = dfdx
	   ereturn matrix se_dfdx = S
	}

	else {
	   ereturn matrix S = S
	}

	ereturn matrix t = t
	ereturn matrix r = r
	ereturn matrix df = df

	ereturn matrix p = p
	ereturn matrix out = out

	ereturn scalar F = `F'
	ereturn scalar pF = `pF'

	ereturn local cmd "`command'"
	ereturn local names `names'
	ereturn local depvar "`depvar'"
        ereturn local converge `convergence'

        if "`goodness'" == "some" {
		ereturn scalar r2 = r2
	}

	ereturn display


}

/* Restore Data */
restore

end

