clear all
set more off

/*********************************** Capital Gap Graphs ***************************************/
use $location/output/tables/accum_capgap_full, clear

* Graph options
local titleoptions "size(medium) color(black) bexpand span"
local displayoptions "graph display, ysize(8) xsize(10)"
local legendoptions "size(medsmall) symy(medium) symx(medium) bmargin(zero) rowgap(.5)"
local labelsize "labsize(small)"
local axistitlesize "size(medium)"
drop if date < $startobs
local labelskip = 8
local endlab = floor(($oldobs - $startobs) / `labelskip') *`labelskip' + $startobs

* Keep thresholds of interest
keep if inlist(target, 5, 8)

* Reshape data
reshape wide gap*, i(date) j(target)

* Sort data
sort date

* Figure 5
twoway line gap5 gap8 date, ///
	$allgraphs ///
	ytitle("Capital Gap ({c $|}bn), 5% and 8% Thresholds", `axistitlesize') ///
	ylabel(0(100)500,`labelsize') ///
	xtitle("", `axistitlesize') ///
	xlabel($startobs(`labelskip')`endlab',`labelsize') ///
	legend(label(1 "5% threshold") label(2 "8% threshold")) ///
	lpattern(shortdash longdash)
graph save "$location/output/figures/figure_5", replace

/******************************* Tier 1 Common Drop Coefficients Graphs ******************************/
local count = 0
local cutoff 2011q1
forvalues i = $startobs/$oldobs {
   disp "Currently Running tq(`i')"
   local scenario "cl_redux"
   use $location/output/tables/capital_forecasts/capital_forecasts_`scenario'_q`i', clear
   qui keep if rank <= 200
   
   qui gen liqsec1 = ((cash - ibb + inv_sec)/cl_assets)*100
   qui gen liqsec2 = ((cash + inv_sec + ffrepo_ass)/cl_assets)*100
   qui gen rwaass = rwa/cl_assets
   sort entity date
   collapse ///
     (first) first_tier1comm = cl_tier1_comm_rat_fc ///
     first_assets = assets ///
     liqsec1 ///
     liqsec2 ///
     rwaass ///
     (last) last_tier1comm = cl_tier1_comm_rat_fc, ///
     by(entity name)
 
     assert _N <= 200

   qui gen tier1comm_drop = last_tier1comm - first_tier1comm
   qui gen first_lnassets = ln(first_assets/1e6)
   
   foreach var of varlist tier1comm_drop first_tier1comm first_lnassets liqsec1 liqsec2 rwaass {
      winsor `var', g(`var'_w) h(2)      
   }
   label var tier1comm_drop "Tier 1 Common: End of Crisis Redux minus Last Historical"
   label var first_tier1comm "Tier 1 Common: Last Historical"
   label var tier1comm_drop_w "Tier 1 Common: End of Crisis Redux minus Last Historical (winsorised at 1%)"
   label var first_tier1comm_w "Tier 1 Common: Last Historical (winsorised at 1%)"
   
   * Capture regression coefficients
      qui reg tier1comm_drop_w first_tier1comm_w
      local c_`i'_first_tier1comm_w = _b[first_tier1comm_w]
      local ccons_`i'_first_tier1comm_w = _b[_cons]
      preserve
      regsave, ci
      qui tempfile ci_`i'_first_tier1comm_w
      qui save `ci_`i'_first_tier1comm_w', replace
      restore
 
   
   qui gen date = `i'
   qui format date %tq
   
   if `count' == 1 {
	qui append using $location/output/tables/PooledRegsData
   }

   local count = 1
   qui save $location/output/tables/PooledRegsData, replace 
}
sort date entity
by date (entity): egen industry_assets = total(first_assets)
gen assetshare = first_assets / industry_assets * 100
gen dummy = (date > tq(`cutoff'))

* Create interactions
gen first_tier1comm_wXdummy = first_tier1comm_w * dummy
gen liqsec2_wXdummy = liqsec2_w * dummy
gen rwaass_wXdummy = rwaass_w * dummy
gen first_lnassets_wXdummy = first_lnassets_w * dummy
save $location/output/tables/PooledRegsData, replace

local rhsvars1 first_tier1comm_w first_tier1comm_wXdummy
local rhsvars2 liqsec2_w liqsec2_wXdummy
local rhsvars3 first_tier1comm_w liqsec2_w first_tier1comm_wXdummy liqsec2_wXdummy
local rhsvars4 rwaass_w rwaass_wXdummy
local rhsvars5 first_lnassets_w first_lnassets_wXdummy
local rhsvars6 first_tier1comm_w liqsec2_w first_lnassets_w first_tier1comm_wXdummy liqsec2_wXdummy first_lnassets_wXdummy
local rhsvars7 first_tier1comm_w liqsec2_w rwaass_w first_lnassets_w first_tier1comm_wXdummy liqsec2_wXdummy rwaass_wXdummy first_lnassets_wXdummy 

* Run pooled regressions weighted by asset share using dummy
putexcel set $location/output/finalized_tables.xlsx, modify sheet("Table2")
forvalues i = 1/7 {
   areg tier1comm_drop_w `rhsvars`i'' [aw=assetshare], absorb(date) vce(cluster entity)
   
   * Observations
   print_info rhsvars`i' observations sigdig table2 e(N)
   putexcel `r(excel_column)'`r(excel_row)' = "`r(formatted_coeff)'"

   * R-squared
   print_info rhsvars`i' r_squared 3 table2 e(r2)
   putexcel `r(excel_column)'`r(excel_row)' = "`r(formatted_coeff)'"
   
   foreach rhsvar of local rhsvars`i' {
		print_info rhsvars`i' `rhsvar' 3 table2 _b[`rhsvar'] _se[`rhsvar']
		putexcel `r(excel_column)'`r(excel_row)' = "`r(formatted_coeff)'`r(stars)'"
		putexcel `r(excel_column)'`=`r(excel_row)'+1' = "(`r(formatted_se)')"	
   }
   
}

* Make Historical SD of Coefficient Graphs
   local count = 1
   clear
   forvalues i = $startobs/$oldobs {
      if `count' == 1 {
         use `ci_`i'_first_tier1comm_w', clear
         qui gen date = `i'
      }
      else {
         append using `ci_`i'_first_tier1comm_w'
         qui replace date = `i' if missing(date)
      }
      local count = 0
   }
   qui drop if var == "_cons"
   format date %tq
   qui tsset date
   qui gen zero = 0
   twoway (rarea ci_upper ci_lower date, sort pstyle(ci) color(gs12)) ///
     (line coef date, lcolor(black) lwidth(medthin)) ///
     (line zero date, lcolor(black) lwidth(thin)), ///
     $allgraphs ///
     xlabel($startobs(4)$oldobs, format("%tq") angle(45)) ///
     xtitle("") ///
	 ytitle("Slope Coefficient", size(medium)) ///
     legend(off)
   qui graph save "$location/output/figures/figure_8", replace
   qui save "$location/output/figures/figure_8", replace

