*****************************************************
/* Run regressions for all firms in the industry */
*****************************************************
* Open quarterly trends data
import excel using "$location/data/raw/quarterly_trends_data.xlsx", sheet("1. Balance Sheet Composition") firstrow clear

* Format stata date
gen year = substr(StataDate,1,4)
gen quarter = substr(StataDate,-1,1)
destring year, replace
destring quarter, replace
gen date = yq(year, quarter)
format %tq date
drop year quarter StataDate
order date

* Merge in ffrepo_ass from class_core_wf
preserve
use "$location/data/class_data/core_wf.dta", clear
keep if entity == 9992000
tempfile class
save `class', replace
restore
merge 1:1 date using `class', keep(3) keepusing(ffrepo_ass) nogen
replace ffrepo_ass = ffrepo_ass / 1000000000

* Keep Assets
keep date Cash InterestBearingBalances Loans SecuritiesExTrading TradingAssets OtherAssets ffrepo_ass

* Consolidate Cash and Interest Bearing Balances
egen cash_ibb = rowtotal(Cash InterestBearingBalances)
drop Cash InterestBearingBalances

* Generate Total Assets Variable
egen total_assets = rowtotal(cash_ibb Loans SecuritiesExTrading TradingAssets OtherAssets ffrepo_ass)

* Convert to percentages
ds date, not
foreach var of varlist `r(varlist)' {
	replace `var' = `var' / total_assets * 100
}

* Restrict to dates we want
keep if date <= tq(2013q3)

* Merge in macro data
merge 1:1 date using "$coredata/macro_log", keep(3) nogen

* Create time trend
gen time = date / 4
replace time = time - 31 //1991q1 is time t = 0

* Create 2009Q1 dummy, when GS and MS enter the sample
gen dummy_09Q1 = date >= tq(2009q1)

* Tsset
tsset date

* Create lags
foreach i of varlist cash_ibb Loans SecuritiesExTrading TradingAssets OtherAssets ffrepo_ass {
   forvalues j = 1/1 {
      gen l`j'_`i' = l`j'.`i'
      label var l`j'_`i' "AR-`j'"
   }
}

* Initialize locals 
local lhsvar1 cash_ibb
local rhsvar1 "dbspread"

local lhsvar2 Loans
local rhsvar2 "spread"

local lhsvar3 SecuritiesExTrading
local rhsvar3 "spread stockgrowth dbspread"

local lhsvar4 TradingAssets
local rhsvar4 "spread dbspread yyhpi"

local lhsvar5 OtherAssets
local rhsvar5 "unchange"

local lhsvar6 ffrepo_ass
local rhsvar6 "dbspread"

local rhsvar "spread stockgrowth dbspread unchange yyhpi"

* Run regressions
* Initialize dataset where we'll print the f-statistics/p-values
tempname memhold
postfile `memhold' regression_num f_test_macro str8 category using "$location/output/tables/f_tests", replace
local outregopt "replace"
local j = 0
forvalues i = 1/6 {
	reg `lhsvar`i'' l1_`lhsvar`i'' `rhsvar' dummy_09Q1 time, robust
	outreg2 using "$location/output/tables/asset_share_specifications", ///
	excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''") `outregopt'
	local outregopt "append"
	
	* Do F-test
	test "`rhsvar'"
	
	* Post to file
	local j = `j' + 1
	post `memhold' (`j') (`r(p)') ("all")
		
	reg `lhsvar`i'' l1_`lhsvar`i'' `rhsvar`i'' dummy_09Q1 time, robust
	outreg2 using "$location/output/tables/asset_share_specifications", ///
	excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''") append
	
	* Do F-test
	test "`rhsvar`i''"
	
	* Post to file
	local j = `j' + 1
	post `memhold' (`j') (`r(p)') ("all")
}
*****************************************************
/* 
 * Run regressions separately for firms in top 10
 * and for firms below the top 10
 */ 
*****************************************************
use "$location/data/class_data/core_wf.dta", clear

* Generate some variables that will be part of the graph
gen loans = ln_tot - llres
gen oth_assets_bs = assets - cash - inv_sec - ffrepo_ass - loans - trad_ass
gen oth_liab_bs = liab_tot - tot_deposit - ffrepo_liab
gen equity_bs = equity + minint_liab

* Generate locals
local assets "oth_assets_bs trad_ass inv_sec loans ffrepo_ass cash"
local liab "equity_bs oth_liab_bs ffrepo_liab tot_deposit"

* Adjust to Trillions
foreach i in `assets' `liab' {
	replace `i' = `i'/1e9
}

preserve
* Remove Aggregates
keep if entity < 9990000

* Identify top 10 in each quarter
gsort date -assets
by date: gen top10 = _n <= 10

* In each quarter, keep the top 10
keep if top10

* Collapse on only top 10
collapse (sum) `assets' `liab', by(date)

* Give this aggregate a name
gen name = "Top 10"

tempfile top10
save `top10', replace
restore

* Keep only the aggregate
keep if entity == 9992000

* Append top10 aggregate
append using `top10'

* Keep only variables of interest
keep date name `assets' `liab'

* Keep only assets
drop `liab'

* Rename names so that we can reshape wide
replace name = "all" if name == "ALL INSTITUTIONS (With Foreigns)"
replace name = "top10" if name == "Top 10"
sort date name

* Reshape wide
ds date name, not
reshape wide `r(varlist)', i(date) j(name) string

* Calculate asset numbers for the complement of the top 10
foreach i in `assets' {
	gen `i'bottom = `i'all - `i'top10
}

* Calculate total assets
egen total_assets_top10 = rowtotal(*top10)
egen total_assets_bottom = rowtotal(*bottom)

* Keep only bottom and top10
drop *all

 * Convert to shares
ds date *bottom, not
foreach var of varlist `r(varlist)' {
	replace `var' = `var' / total_assets_top10 * 100
}

ds date *top10, not
foreach var of varlist `r(varlist)' {
	replace `var' = `var' / total_assets_bottom * 100
}

tempfile compile
save `compile', replace

* Merge in macro data
merge 1:1 date using "$coredata/macro_log", keep(3) nogen

* Create time trend
gen time = date / 4
replace time = time - 31 //1991q1 is time t = 0

* Create 2009Q1 dummy, when GS and MS enter the sample
gen dummy_09Q1 = date >= tq(2009q1)

* Tsset
tsset date

* Create lags
foreach i in `assets' {
   forvalues j = 1/1 {
      gen l`j'_`i'top10 = l`j'.`i'top10
      label var l`j'_`i'top10 "AR-`j' top10"
      
      gen l`j'_`i'bottom = l`j'.`i'bottom
      label var l`j'_`i'bottom "AR-`j' bottom"
   }
}

* Initialize locals 
local lhsvar1 cash
local rhsvar1 "dbspread"

local lhsvar2 loans
local rhsvar2 "spread"

local lhsvar3 inv_sec
local rhsvar3 "spread stockgrowth dbspread"

local lhsvar4 trad_ass
local rhsvar4 "spread dbspread yyhpi"

local lhsvar5 oth_assets_bs
local rhsvar5 "unchange"

local lhsvar6 ffrepo_ass
local rhsvar6 "dbspread"

local rhsvar "spread stockgrowth dbspread unchange yyhpi"

* Run regressions
foreach group in top10 bottom {
	local outregopt "replace"
	local j = 0
	forvalues i = 1/6 {
		reg `lhsvar`i''`group' l1_`lhsvar`i''`group' `rhsvar' dummy_09Q1 time, robust
		outreg2 using "$location/output/tables/asset_share_specifications_`group'", ///
		excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''`group'") `outregopt'
		local outregopt "append"
		
		* Do F-test
		test "`rhsvar'"
		
		* Post to file
		local j = `j' + 1
		post `memhold' (`j') (`r(p)') ("`group'")
		
		reg `lhsvar`i''`group' l1_`lhsvar`i''`group' `rhsvar`i'' dummy_09Q1 time, robust
		outreg2 using "$location/output/tables/asset_share_specifications_`group'", ///
		excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''`group'") append
			
		* Do F-test
		test "`rhsvar`i''"
		
		* Post to file
		local j = `j' + 1
		post `memhold' (`j') (`r(p)') ("`group'")
	}
}

* Close post
postclose `memhold'

* Open post file and format to print to reg tables
use "$location/output/tables/f_tests", clear

* Reshape wide
reshape wide f_test_macro, i(category) j(regression_num)

* Save formatted f-tests dataset
save "$location/output/tables/f_tests", replace
export excel using "$location/output/tables/f_tests.xlsx", firstrow(var) replace

*****************************************************
/* 
 * Run pooled regressions with top10 and bottom, 
 * and use interaction terms to do F-tests.
 */ 
*****************************************************
* Do pooled regressions with top10 and bottom, and use interaction terms to do F-tests
use `compile', clear

* Reshape long
drop total_assets*
reshape long `assets', i(date) j(type) string

* Create indicators for "small" banks
gen small = (type == "bottom")

* Merge in macro data
merge m:1 date using "$coredata/macro_log", keep(3) nogen

* Create time trend
gen time = date / 4
replace time = time - 31 //1991q1 is time t = 0

* Create 2009Q1 dummy, when GS and MS enter the sample
gen dummy_09Q1 = date >= tq(2009q1)

* Tsset
tsset small date

* Initialize locals 
local lhsvar1 cash
local rhsvar1 "dbspread"
local rhsvar1_interact "c.dbspread"

local lhsvar2 loans
local rhsvar2 "spread"
local rhsvar2_interact "c.spread"

local lhsvar3 inv_sec
local rhsvar3 "spread stockgrowth dbspread"
local rhsvar3_interact "c.spread c.stockgrowth c.dbspread"

local lhsvar4 trad_ass
local rhsvar4 "spread dbspread yyhpi"
local rhsvar4_interact "c.spread c.dbspread c.yyhpi"

local lhsvar5 oth_assets_bs
local rhsvar5 "unchange"
local rhsvar5_interact "c.unchange"

local lhsvar6 ffrepo_ass
local rhsvar6 "dbspread"
local rhsvar6_interact "c.dbspread"

local rhsvar "spread stockgrowth dbspread unchange yyhpi"
local rhsvar_interact "c.spread c.stockgrowth c.dbspread c.unchange c.yyhpi"

* Store in a local all the interacted macrovariables, to do an F-test
local test_vars = ""
foreach type in `rhsvar_interact' {
	local test_vars = "`test_vars' c.small#`type'"
}

* Run regressions and open Post File
tempname memhold
postfile `memhold' regression_num f_test_macro str8 category using "$location/output/tables/f_tests_interaction", replace
local j = 0
local outregopt "replace"
forvalues i = 1/6 {
	reg `lhsvar`i'' l1.`lhsvar`i'' c.l1.`lhsvar`i''#c.small `rhsvar' (`rhsvar_interact')#c.small dummy_09Q1 c.dummy_09Q1#c.small time c.time#c.small, robust
	outreg2 using "$location/output/tables/asset_share_specifications_interaction", ///
	excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''") `outregopt'
	local outregopt "append"
	
	* Do F-test
	test "`test_vars'"
	
	* Post to file
	local j = `j' + 1
	post `memhold' (`j') (`r(p)') ("interaction")
	
	reg `lhsvar`i'' l1.`lhsvar`i'' c.l1.`lhsvar`i''#c.small `rhsvar`i'' (`rhsvar`i'_interact')#c.small dummy_09Q1 c.dummy_09Q1#c.small time c.time#c.small, robust
	outreg2 using "$location/output/tables/asset_share_specifications_interaction", ///
	excel nor2 label addstat(R^2, e(r2), Adj R^2, e(r2_a), F-stat, e(F)) ctitle("`lhsvar`i''") `outregopt'
	
	* Store in a local all the interacted macrovariables in the parsimonious specification, to do an F-test
	local pars_vars = ""
	foreach type in `rhsvar`i'_interact' {
		local pars_vars = "`pars_vars' c.small#`type'"
	}

	* Do F-test
	test "`pars_vars'"
	
	* Post to file
	local j = `j' + 1
	post `memhold' (`j') (`r(p)') ("interaction")
}

* Close post
postclose `memhold'

* Open post file and format to print to reg tables
use "$location/output/tables/f_tests_interaction", clear

* Reshape wide
reshape wide f_test_macro, i(category) j(regression_num)

* Save formatted f-tests dataset
save "$location/output/tables/f_tests_interaction", replace
export excel using "$location/output/tables/f_tests_interaction.xlsx", firstrow(var) replace
