* CLASS Regressions

clear all
set more off

cd "$location"

/*************** 1. Merge Bank Data and Macro Data, save Industry-level historical data *******************/
use "$coredata/core$datasuffix", clear

* Fix AFS Returns data, setting = 0 if missing
replace cl_afs_return = 0 if missing(cl_afs_return)

* Create new variables for later graphs
gen cl_ppnr_ass = ( cl_ppnr / cl_assets ) * 400
gen cl_nierat = ( qnonint_exp / cl_assets ) * 400
gen cl_equity_rat = ( equity / rwa ) * 100
gen cl_netinc_assets = ( qnetinc / cl_assets ) * 400
gen cl_payout_rat = ( qprefdividend + qcommdividend ) / qnetinc * 100
replace cl_payout_rat = 100 if ( cl_payout_rat > 100 | qnetinc < 0 )
gen cl_tier1_comm_rat = (tier1_comm / rwa) * 100 if date >= tq(2001q1)
gen cl_liab_tot = liab_tot

* Merge with macro_log
capture drop _merge
merge m:1 date using "$coredata/macro_log.dta", nogen keep(match)

sort entity date
tsset entity date

* Remove CCAR firms
drop if entity == 9990090

/********************* 2. Create Lags and Portfolio Shares ********************/
foreach i of varlist cl_ppnr_rat cl_nim cl_nintrat cl_tradrat cl_nierat cl_nie_comp ///
					 cl_nie_fass cl_nie_allother cl_nco_ci cl_nco_cc cl_nco_othcons ///
					 cl_nco_lease cl_nco_allother cl_nco_dep cl_nco_agr cl_nco_fgovt ///
					 cl_nco_oth cl_nco_rre cl_nco_heloc cl_nco_jrlien cl_nco_firstlien ///
					 cl_nco_cre cl_nco_multi cl_nco_const cl_nco_nfnr cl_nco_othre cl_afs_return {
   forvalues j = 1/1 {
      gen l`j'_`i' = l`j'.`i'
      label var l`j'_`i' "AR-`j'"
   }
}

* Create loan portfolio shares:
foreach x in rre cre ci cc {
	gen rat_`x' = (ln_`x'/cl_intearn_ass)*100
	replace rat_`x'=0 if ln_`x'==0
}
gen rat_trad = (cl_trading_ass/cl_intearn_ass)*100
gen rat_inv = (inv_sec/cl_intearn_ass)*100

* Create asset shares and some additional transformations
gen d10y = cpr_t10y - l1.cpr_t10y
gen dbspread_pos = dbspread * (dbspread > 0)   
gen unsafe_dbspread = unsafe_ratio*dbspread
gen unsafe_dbspread_pos = unsafe_dbspread * (dbspread > 0)   

bys date: egen sum_assets = total(cl_assets) if (rank<202|entity==9990001)
gen assetshare = (cl_assets / sum_assets)*100
bys date: egen total_intearn_assets = sum(cl_intearn_ass) if (rank<202|entity==9990001)
gen intearn_assetshare = (cl_intearn_ass / total_intearn_assets)*100
bys date: egen total_afssec = sum(afssec) if (rank<202|entity==9990001)
gen afsshare = (afssec / total_afssec)*100

* Adjust time trend
** 1991q1 is initially equal to 31, but we would like it to be set at 0
gen time = date / 4
replace time = time - 31
qui summ date
local end_date = r(max)

* Labeling revenue components 
label var cl_nim "Net Interest Margin" 
label var cl_nintrat "Noninterest Non-Trading Income Ratio"
label var cl_tradrat "Trading Income Ratio"
label var cl_nie_comp "Compensation Expense Ratio"
label var cl_nie_fass "Fixed Asset Expense Ratio"
label var cl_nie_allother "All Other Expense Ratio"
label var cl_ppnr_ass "PPNR ratio" 

* Labeling macro and asset ratios 
label var spread "Term Spread (10Y-3M)"
label var growth "Real GDP Annualized Log Change"
label var stockgrowth "Stock Market Quarterly Log Change" 
label var cpr_t3m "3M Treasury Yield" 
label var cpr_t10y "10Y Treasury Yield"
label var dbspread "Change in bond spread"
label var dbspread_pos "Change in bond spread X >0 Dummy" 
label var unsafe_dbspread_pos "AFS Ratio X Change in bond spread X >0 Dummy" 
label var unchange "Change in Unemployment"
label var yyhpi "Year over year HPI" 
label var yyhpi_zero "Year over year HPI X <0 Dummy" 
label var yycppi "Year over year CPPI"
label var yycppi_zero "Year over year CPPI X <0 Dummy" 
label var rat_rre "Residential RE Loan Ratio"
label var rat_cre "Commercial RE Loan Ratio"
label var rat_ci "C&I Loan Ratio"
label var rat_cc "Credit Card Loan Ratio"
label var rat_trad "Trading Assets Ratio" 
label var rat_inv "Securities Ratio" 
label var assetshare "Asset share"
label var time "Time"

label var cl_nco_rre "Total NCO RRE" 
label var cl_nco_firstlien "NCO First Lien RRE"
label var cl_nco_jrlien "NCO Junior Lien RRE"
label var cl_nco_heloc "NCO HELOC RRE"
label var cl_nco_cre "Total NCO CRE" 
label var cl_nco_const "NCO Construction CRE" 
label var cl_nco_multi "NCO Multifamily CRE"
label var cl_nco_nfnr "NCO Non-Farm Non-Res CRE"
label var cl_nco_cc "NCO Credit Card" 
label var cl_nco_othcons "NCO Other Consumer"
label var cl_nco_ci "NCO C&I" 
label var cl_nco_lease "NCO Leases"
label var cl_nco_allother "NCO Total All Other"
label var cl_nco_othre "NCO Other Real Estate" 
label var cl_nco_dep "NCO Other Depository Institutions"
label var cl_nco_agr "NCO Agriculture"
label var cl_nco_fgovt "NCO Foreign Governments"
label var cl_nco_oth "NCO All Other" 

/******************** Specifications: Revenue/AFS Securities **********************/

local regtype "reg" 
local robustoption "robust" 
local weight "pweight"
local option "cluster(time)"

local lhsvar1 nim /* Net Interest Income / Interest Earning Assets */
local rhsvars1 "spread cpr_t3m rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare time"
local options1 "(rank<201 | entity==9990001) & date>=tq(1991q1) [`weight'=intearn_assetshare], `option'" 
local lhsvar2 nintrat /* Non-Interest Non-Trading Income / Total Assets (Annualized) */
local rhsvars2 "stockgrowth rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare"
local options2 "(rank<201 | entity==9990001) & date>=tq(1991q1) [`weight'=assetshare], `robustoption'"
local lhsvar3 tradrat /* Trading Revenue / Trading Assets */
local rhsvars3 "dbspread dbspread_pos"
local options3 "entity==9991000 & date>tq(1996q4), `robustoption'"

local lhsvar4 nie_comp /* Non-Interest Expense on Compensation / Total Assets (Annualized) */
local rhsvars4 "stockgrowth rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare"
local options4 "(rank<201 | entity==9990001) & date>=tq(1991q1) [`weight'=assetshare], `option'"
local lhsvar5 nie_fass /* Non-Interest Expense on Fixed Assets / Total Assets (Annualized) */
local rhsvars5 "time growth rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare"
local options5 "(rank<201 | entity==9990001) & date>=tq(1991q1) [`weight'=assetshare], `option'" 
local lhsvar6 nie_allother /* Non-Interest Expense on Goodwill, Amort, and Other / Total Assets (Annualized) */
local rhsvars6 "dbspread rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare"
local options6 "(rank<201 | entity==9990001) & date>=tq(1991q1) [`weight'=assetshare], `option'"
local lhsvar7 afs_return /* Gain on AFS Securities / Total AFS Securities (Annualized) */
local rhsvars7 "d10y unsafe_dbspread_pos"
local options7 "(rank<201 | entity==9990001) & date>=tq(1997q1) [`weight'=afsshare], `option'"

local outregopt "replace"
forvalues i = 1/7 {

   `regtype' cl_`lhsvar`i'' l1_cl_`lhsvar`i'' `rhsvars`i'' if `options`i''
   estimates save $location/data/$folder/cl_`lhsvar`i'', replace
}

/******************* Specifications: Net Charge-Offs *********************/

local regtype "cnsreg"
local conoption "constraints(1)"
local robustoption ""

local lhsvar8 nco_rre
local rhsvars8 "yyhpi yyhpi_zero"
local options8 "entity==9991000 & date>=tq(1991q1)"
local lhsvar9 nco_firstlien
local rhsvars9 "yyhpi yyhpi_zero"
local options9 "entity==9991000 & date>=tq(1991q1)"
local lhsvar10 nco_jrlien
local rhsvars10 "yyhpi yyhpi_zero"
local options10 "entity==9991000 & date>=tq(1991q1)"
local lhsvar11 nco_heloc
local rhsvars11 "yyhpi yyhpi_zero"
local options11 "entity==9991000 & date>=tq(1991q1)"
local lhsvar12 nco_cre
local rhsvars12 "yycppi_zero"
local options12 "entity==9991000 & date>=tq(1991q1)"
local lhsvar13 nco_const
local rhsvars13 "yycppi_zero"
local options13 "entity==9991000 & date>=tq(1991q1)"
local lhsvar14 nco_multi
local rhsvars14 "yycppi_zero"
local options14 "entity==9991000 & date>=tq(1991q1)"
local lhsvar15 nco_nfnr
local rhsvars15 "yycppi_zero"
local options15 "entity==9991000 & date>=tq(1991q1)"
local lhsvar16 nco_cc
local rhsvars16 "unchange"
local options16 "entity==9991000 & date>=tq(1991q1)"
local lhsvar17 nco_othcons
local rhsvars17 "unchange time"
local options17 "entity==9991000 & date>=tq(1991q1)"
local lhsvar18 nco_ci
local rhsvars18 "unchange"
local options18 "entity==9991000 & date>=tq(1991q1)"
local lhsvar19 nco_lease
local rhsvars19 "unchange"
local options19 "entity==9991000 & date>=tq(1991q1)"
local lhsvar20 nco_allother
local rhsvars20 "unchange"
local options20 "entity==9991000 & date>=tq(1994q1)"
local lhsvar21 nco_othre
local rhsvars21 "yycppi yycppi_zero"
local options21 "entity==9991000 & date>=tq(1991q1)"
local lhsvar22 nco_dep
local rhsvars22 "unchange"
local options22 "entity==9991000 & date>=tq(1991q1)"
local lhsvar23 nco_agr
local rhsvars23 "unchange"
local options23 "entity==9991000 & date>=tq(1991q1)"
local lhsvar24 nco_fgovt
local rhsvars24 "unchange"
local options24 "entity==9991000 & date>=tq(1991q1)"
local lhsvar25 nco_oth
local rhsvars25 "unchange"
local options25 "entity==9991000 & date>=tq(1991q1)"

local outregopt "replace"
forvalues i = 8/25 {
      reg cl_`lhsvar`i'' l1_cl_`lhsvar`i'' `rhsvars`i'' if `options`i''
      local constraintvalue = min(_b[l1_cl_`lhsvar`i''],.99)
      constraint 1 l1_cl_`lhsvar`i'' = `constraintvalue'
      disp "Constraint = `constraintvalue'"
   
   `regtype' cl_`lhsvar`i'' l1_cl_`lhsvar`i'' `rhsvars`i'' if `options`i'', `robustoption' `conoption'
   estimates save $location/data/$folder/cl_`lhsvar`i'', replace
}

exit, clear
