* this program takes the regressions estimated in 2regs.do, and uses them to create forecasts of capital, net chargeoffs etc.
 
clear all
set more off

cd "$location"

* 1. Parameters and toggles
* Horizon for projections (in quarters): note this is four quarters larger than $horizon because of 4q-ahead nco projection
local horizon = $horizon + 4

* 2. Generate variables
use "$coredata/capital$datasuffix", clear
keep if date == $oldobs

* This bank has a "/" in its name, so replace that
replace name = "CULLEN FROST BKR" if name == "CULLEN/FROST BKR"

* Drop particular institutions [all institutions, ICE and DTC]
drop if (entity == 52719 | entity == 3822016)

* Generate new variables
gen cl_dividend = qprefdividend + qcommdividend
gen cl_ppnr_ass = cl_ppnr / cl_assets
gen cl_nierat = ( qnonint_exp / cl_assets ) * 400
gen qnonint_inc_nt = qnonint_inc - qtradrev_inc
gen pretax = cl_ppnr - qllprov + qafssecur_inc + qhtmsecur_inc 
qui gen cl_tier1_comm_rat = tier1_comm / rwa * 100

* Replace missing data with zeros
foreach i in tot cre rre allother ci cc othcons heloc firstlien jrlien const ///
		     multi nfnr othre lease fgovt oth agr dep {
	qui replace qnetxoff_`i' = 0 if qnetxoff_`i' == .
}

* Create portfolio shares for interest-earning assets:
foreach x in rre cre ci cc {
   	gen rat_`x'=(ln_`x'/cl_intearn_ass)*100
   	replace rat_`x'=0 if ln_`x'==0
}
gen rat_inv = (inv_sec/cl_intearn_ass)*100
gen rat_trad = (cl_trading_ass/cl_intearn_ass) * 100

* Create asset shares by institution
bys date: egen sum_assets = total(cl_assets) if (rank!=999 | entity==9990001)
bys date: gen assetshare = (cl_assets / sum_assets)*100 
bys date: egen sum_rwa = total(rwa) if (rank!=999 | entity==9990001)
bys date: gen rwashare = (rwa / sum_rwa)*100
bys date: egen total_intearn_assets = sum(cl_intearn_ass) if (rank!=999 | entity==9990001)
bys date: gen intearn_assetshare = (cl_intearn_ass / total_intearn_assets)*100
bys date: egen total_afssec = sum(afssec) if (rank!=999 | entity==9990001)
bys date: gen afsshare = (afssec / total_afssec)*100

* Adjust time trend
** 1991q1 is initially equal to 31; we reset it to be equal to 0
local end_date = r(max)
gen time = date / 4
replace time = time - 31

* Generate deferred tax asset variable
replace t1_netdef_taxassets = 0 if t1_netdef_taxassets == .
gen dta = max(t1_netdef_taxassets - t1_netdef_taxliab, 0)
gen dta_dis_tot = t1_disallowed_tax_ass
replace dta_dis_tot = 0 if missing(t1_disallowed_tax_ass)
gen dta_allow_tot = dta - dta_dis_tot
gen dta_allow_past = max(dta_allow_tot - 0.1 * t1_regcap_subtot, 0)
gen dta_allow_fut = dta_allow_tot - dta_allow_past
gen dta_total_fut = dta_allow_fut + dta_dis_tot
gen dta_check = dta_total_fut + dta_allow_past

* 3. Tsset data, merge in macro data and create lags
sort entity date
tsset entity date

foreach i of varlist cl_ppnr_rat cl_nim cl_nintrat cl_tradrat cl_nierat cl_nie_comp ///
				     cl_nie_fass cl_nie_allother cl_nco_ci cl_nco_cc cl_nco_othcons ///
					 cl_nco_lease cl_nco_allother cl_nco_dep cl_nco_agr cl_nco_fgovt ///
					 cl_nco_oth cl_nco_heloc cl_nco_jrlien cl_nco_firstlien cl_nco_multi ///
					 cl_nco_const cl_nco_nfnr cl_nco_othre cl_afs_return {
		gen l1_`i' = l1.`i'
}

* Merge with macro data
sort date
capture drop _merge
merge date using "$coredata/macro_log.dta"
keep if _merge == 3
drop _merge

gen temp = (entity[_n] ~= entity[_n-1])
gen count = sum(temp)
local number = count[_N]
qui drop temp

* Create macro variables and interaction terms between BHC and macro data
sort entity date
gen d10y = cpr_t10y - l1.cpr_t10y
gen dbspread_pos = dbspread * (dbspread > 0)   
gen unsafe_dbspread = unsafe_ratio * dbspread
gen unsafe_dbspread_pos = unsafe_dbspread * (dbspread > 0)   

* 4. Generate forecasts by BHC / bank
foreach scenario in $scenarios {
   local firstbank = 0
   forvalues bank = 1/`number' {
      qui preserve
      
      * Keep only the institution you are analyzing: set bank dummy equal to its final value
      qui keep if count == `bank'
      
      * Set parameters
      qui local reserve = 0
      qui local sample = _N
      qui local oldobs = `sample' - `reserve'
      qui local newobs = `sample' + `horizon' - `reserve'
      
      * Create a name for each institution, and increase observations to end of sample period
      qui local institution = name[`oldobs']
      qui sort date
      qui set obs `newobs'
      
      dis "-------------------------------------------------------------------------------"
      dis "  count = `bank'  and  name = `institution' and scenario = `scenario'"
      dis "-------------------------------------------------------------------------------"
      
      * Update the date variable. Keep time trend fixed over forecast horizon.
      qui replace date = date[`oldobs'] + _n - `oldobs' if _n > `oldobs'
	  * Adjust time trend: set 1991q1 equal to 0 rather than 31
      qui replace time = min(date,date[`oldobs']) / 4 - 31
      
      * Decide which macro scenario you are interested in, and merge in the appropriate macro data
      qui sort date
      qui merge date using "$coredata/fcasts_`scenario'.dta"
      qui keep if _merge == 3
      qui tsset date
      
      foreach macro of varlist spread cpr_t3m dbspread dbspread_pos growth stockgrowth ///
	                           yyhpi yycppi yyhpi_zero yycppi_zero unchange d10y {
         qui replace `macro' = f_`macro' if _n > `sample'
	  }
      
      * Keep "denominator" variables constant over the forecast horizon
      foreach x of varlist rat_rre rat_cre rat_ci rat_cc rat_trad rat_inv assetshare ///
	                       rwashare intearn_assetshare afsshare {
         qui replace `x' = `x'[`oldobs'] if _n > `oldobs'
      }
      
      * Keep share of unsafe assets constant over forecast horizon [note: set to zero for small banks]
      qui replace unsafe_ratio = 0 if entity == 9990001
      qui replace unsafe_ratio = unsafe_ratio[`oldobs'] if _n > `oldobs'
      replace unsafe_dbspread = unsafe_ratio * dbspread
      replace unsafe_dbspread_pos = unsafe_dbspread * (dbspread > 0)   
      
      * JPMC 2013q3 adjustment
	  ** Accounting for large litigation expense charge in this quarter (non-autoregressive)
      if entity == 1039502 & $loopqtr == tq(2013q3) {
         local jpm_adjustment = (8758500 / cl_assets[`oldobs']) * 400
         dis "JPM adjustment:  `jpm_adjustment'"
         replace cl_nie_allother = cl_nie_allother - `jpm_adjustment' in `oldobs'
	  }

      * RBS 2013q2 adjustment
	  ** Accounting for large goodwill impairment losses in this quarter (non-autoregressive)
      if entity == 1132449 & $loopqtr == tq(2013q2) {
         local rbs_adjustment = (4435000 / cl_assets[`oldobs']) * 400
         dis "RBS adjustment:  `rbs_adjustment'"
         replace cl_nie_allother = cl_nie_allother - `rbs_adjustment' in `oldobs'
	  }
      
      * Bank of America 2011q2 adjustment 
	  ** Accounting for large mortgage charge in this quarter (non-autoregressive)
      if entity == 1073757 & $loopqtr == tq(2011q2) {
         local boa_adjustment = (12800000 / cl_assets[`oldobs']) * 400
         dis "Bank of America adjustment:  `boa_adjustment'"
         replace cl_nintrat = cl_nintrat + `boa_adjustment' in `oldobs'
	  }
      
      * Compensation and All Other Expenses 2011q3 adjustment 
      replace cl_nie_comp = cl_nie_comp + (0.123 * ($loopqtr == tq(2011q3))) in `oldobs'
      replace cl_nie_allother = cl_nie_allother + (0.123 * ($loopqtr == tq(2011q3))) in `oldobs'
      
	  * Produce recursive forecasts for each ratio
      foreach measure in cl_nim cl_tradrat cl_nintrat cl_nie_comp cl_nie_fass ///
	  cl_nie_allother cl_nco_heloc cl_nco_firstlien cl_nco_jrlien cl_nco_const ///
	  cl_nco_multi cl_nco_nfnr cl_nco_othre cl_nco_dep cl_nco_agr cl_nco_fgovt ///
	  cl_nco_oth cl_nco_ci cl_nco_cc cl_nco_othcons cl_nco_lease cl_afs_return  {
         estimates use $location/data/$folder/`measure'
         qui gen `measure'_h = `measure'
         forvalues i = 1/`horizon' {
            qui local j = `oldobs' + `i'
            qui replace l1_`measure' = `measure'[`oldobs'+`i'-1] in `j'
            qui predict temp in `j'
            qui replace `measure' = temp in `j'
            qui drop temp
         }
      }
    
      qui replace entity = entity[`sample'] if _n > `sample'
      qui replace rank = rank[`sample'] if _n > `sample'
      
      * Create dollar value forecast variables, to fill in below
      foreach i in qint_inc_net qtradrev_inc qnonint_inc_nt qnonint_inc qnonint_exp_comp ///
	  qnonint_exp_fass qnonint_exp_allother qnonint_exp cl_ppnr cl_ppnr_ass qafssecur_inc ///
	  qhtmsecur_inc llres qllprov pretax qtaxes_inc qminor_int qextra_inc qnetinc ///
	  equity t1_regcap_subtot dta dta_allow_past dta_total_fut dta_allow_fut dta_dis_tot ///
	  liab_tot tier1_rbc cl_dividend tier1_comm rwa cl_tier1_rat cl_tier1_comm_rat {
         qui gen `i'_fc = `i' in `oldobs'
	  }
		
      foreach i in tot cre rre allother ci cc othcons heloc firstlien jrlien const ///
	               multi nfnr othre lease fgovt oth agr dep {
         qui gen qnetxoff_`i'_fc = qnetxoff_`i' in `oldobs'
         qui gen fourqtr_nco_`i'_fc = .
	  }
      
      * Save unadjusted versions of loan loss reserves and equity items
      foreach i in llres equity tier1_comm tier1_rbc cl_tier1_rat cl_tier1_comm_rat {
         qui gen `i'_unadj = `i' in `oldobs'
	  }
      
      qui replace fourqtr_nco_tot_fc = 0
      foreach i in ci cc othcons heloc firstlien jrlien const multi nfnr othre ///
	               lease fgovt oth agr dep {
         qui replace fourqtr_nco_`i'_fc = 0.25 * (cl_nco_`i'[`oldobs'+1] + cl_nco_`i'[`oldobs'+2] + cl_nco_`i'[`oldobs'+3] + cl_nco_`i'[`oldobs'+4])/ 100 * ln_`i'[`oldobs'] in `oldobs'
         qui replace fourqtr_nco_tot_fc = fourqtr_nco_tot_fc + fourqtr_nco_`i'_fc in `oldobs'
	  }
      
	  if "${provisions}" == "four_smooth" {
		 local trueup = fourqtr_nco_tot_fc - llres_unadj in `oldobs'
	  }

	  if "${provisions}" == "four" {
		 qui replace llres = fourqtr_nco_tot_fc in `oldobs'
	  }
	  
	  if "${provisions}" == "tunnel" {
		 qui gen lowerbound = fourqtr_nco_tot_fc in `oldobs'
		 qui gen upperbound = 2.5 * lowerbound in `oldobs'
		 qui replace llres = upperbound if llres >= upperbound in `oldobs'
		 qui replace llres = lowerbound if llres <= lowerbound in `oldobs'
	  }
		
	  if "${provisions}" == "tunnel_smooth" {
		 qui gen lowerbound = fourqtr_nco_tot_fc in `oldobs'
		 qui gen upperbound = 2.5 * lowerbound in `oldobs'
		 
		 if llres[`oldobs'] >= upperbound[`oldobs'] {
			local trueup =  upperbound - llres_unadj in `oldobs' 
		 }
		 if llres[`oldobs'] <= lowerbound[`oldobs'] {
			local trueup =  lowerbound - llres_unadj in `oldobs' 
		 }
		 if llres[`oldobs'] > lowerbound[`oldobs'] & llres[`oldobs'] < upperbound[`oldobs'] {
			local trueup = 0
		 }
	  }
	
      qui replace equity = equity - (llres - llres_unadj) in `oldobs'
      qui replace tier1_comm = tier1_comm - (llres - llres_unadj) in `oldobs'
      qui replace tier1_rbc = tier1_rbc - (llres - llres_unadj) in `oldobs'
      qui replace cl_tier1_rat = tier1_rbc / rwa * 100 in `oldobs' 
      qui replace cl_tier1_comm_rat = tier1_comm / rwa * 100 in `oldobs'
      
      foreach i in llres equity tier1_comm tier1_rbc cl_tier1_rat cl_tier1_comm_rat {
		qui replace `i'_fc = `i' in `oldobs'
	  }
            
      * Run a loop that fills in the forecast balances
      forvalues i = 1/`horizon' {
         qui local j = `oldobs' + `i'
         
         foreach balance in cl_assets cl_intearn_ass cl_trading_ass rwa afssec ///
		 htmsec ln_tot ln_re ln_rre ln_firstlien ln_jrlien ln_heloc ln_closedlien ///
		 ln_cre ln_const ln_multi ln_nfnr ln_othre ln_dep ln_agr ln_ci ln_cc ///
		 ln_othcons ln_fgovt ln_oth ln_lease ln_allother ln_tot {
            qui replace `balance' = `balance'[`j'-1] * $assetmultiplier in `j'
            qui replace `balance' = 0 if (`balance' == .)
		 }
         
         * Calculate dollar projections for components of PPNR
         qui replace qint_inc_net_fc = cl_nim * cl_intearn_ass / 400 in `j'
         qui replace qtradrev_inc_fc = cl_tradrat * cl_trading_ass / 400 in `j'
         qui replace qnonint_inc_nt_fc = cl_nintrat / 400 * cl_assets in `j'
         qui replace qnonint_inc_fc = qnonint_inc_nt_fc + qtradrev_inc_fc in `j'
         qui replace qnonint_exp_comp_fc = cl_nie_comp / 400 * cl_assets in `j'
         qui replace qnonint_exp_fass_fc = cl_nie_fass / 400 * cl_assets in `j'
         qui replace qnonint_exp_allother_fc = cl_nie_allother / 400 * cl_assets in `j'
         qui replace qnonint_exp_fc = qnonint_exp_comp_fc + qnonint_exp_fass_fc + qnonint_exp_allother_fc in `j'
         qui replace cl_ppnr_fc = qint_inc_net_fc + qnonint_inc_fc - qnonint_exp_fc in `j'
         qui replace cl_ppnr_ass_fc = cl_ppnr_fc / cl_assets in `j'
         
         * Calculate dollar projections for AFS/HTM portfolio
         qui replace qafssecur_inc_fc = cl_afs_return / 400 * afssec in `j'
			* Assume return on HTM is 0
			qui replace qhtmsecur_inc_fc = 0 in `j'
         
         * Calculate dollar projections for quarterly net chargeoffs 
		 ** Note: must be divided by four because NCO ratio is annualized
         foreach i in ci cc othcons heloc firstlien jrlien const multi nfnr othre lease fgovt oth agr dep {
            qui replace qnetxoff_`i'_fc = 0.25 * (cl_nco_`i' / 100) * ln_`i' in `j'
		 }
         qui sort date
         
         * Sum up previous values to larger loan categories
         qui replace qnetxoff_allother_fc = qnetxoff_othre_fc + qnetxoff_fgovt_fc + qnetxoff_dep_fc + qnetxoff_agr_fc + qnetxoff_oth_fc in `j'
         qui replace qnetxoff_cre_fc = qnetxoff_const_fc + qnetxoff_multi_fc + qnetxoff_nfnr_fc in `j'  
         qui replace qnetxoff_rre_fc = qnetxoff_firstlien_fc + qnetxoff_jrlien_fc + qnetxoff_heloc_fc in `j'
         qui replace qnetxoff_tot_fc = qnetxoff_ci_fc + qnetxoff_cc_fc + qnetxoff_othcons_fc + qnetxoff_rre_fc + qnetxoff_cre_fc + qnetxoff_lease_fc + qnetxoff_allother_fc in `j'
         
         * Make cl_ncotype_fc variables for aggregates
         foreach ncotype in allother cre rre tot {
            capture gen cl_nco_`ncotype'
            qui replace cl_nco_`ncotype' = 400*(qnetxoff_`ncotype'_fc/ln_`ncotype') in `j'
         }
         
         * Provisions: calculate 4Q measure of future chargeoffs based on current asset balances
         foreach i in ci cc othcons heloc firstlien jrlien const multi nfnr othre lease fgovt oth agr dep {
            qui replace fourqtr_nco_`i'_fc = 0.25 * (cl_nco_`i'[`j'+1] + cl_nco_`i'[`j'+2] + cl_nco_`i'[`j'+3] + cl_nco_`i'[`j'+4])/ 100 * ln_`i'[`j'] in `j'
            qui replace fourqtr_nco_tot_fc = fourqtr_nco_tot_fc + fourqtr_nco_`i'_fc in `j'
		 }
         
         * Provisions: compute provisions and loan loss reserves based on each of the five methods we use
         if "${provisions}" == "NCO" {
            qui replace llres_fc = llres[`oldobs'] in `j'
		 }
			
	     if "${provisions}" == "four_smooth" {
			local step = max(1-(`j'-1)/${prov_adj_period},0)
			qui replace llres_fc = fourqtr_nco_tot_fc - (`trueup' * `step') in `j'
		 }
         
         if "${provisions}" == "four" {
            qui replace llres_fc = fourqtr_nco_tot_fc in `j'
		 }
		
		 if "${provisions}" == "tunnel_smooth" {
 			local step = max(1-(`j'-1)/${prov_adj_period},0)
			local adj = `trueup'/${prov_adj_period}
            qui replace llres_fc = l1.llres_fc + `adj'*(`j' <= (${prov_adj_period} + 1)) in `j'
            qui replace lowerbound = fourqtr_nco_tot_fc - (`trueup' * `step') in `j'
            qui replace upperbound = 2.5 * lowerbound in `j'
            qui replace llres_fc = upperbound[`j'] if llres_fc >= upperbound in `j'
            qui replace llres_fc = lowerbound[`j'] if llres_fc <= lowerbound in `j'
		 }
         
         if "${provisions}" == "tunnel" {
            qui replace llres_fc = l1.llres_fc in `j'
            qui replace lowerbound = fourqtr_nco_tot_fc in `j'
            qui replace upperbound = 2.5 * lowerbound in `j'
            qui replace llres_fc = upperbound[`j'] if llres_fc >= upperbound in `j'
            qui replace llres_fc = lowerbound[`j'] if llres_fc <= lowerbound in `j'
		 }
			
         qui replace qllprov_fc = qnetxoff_tot_fc + (llres_fc - l1.llres_fc) in `j'
         
         qui sort date
         qui tsset date
         qui local first = `oldobs' + 1
			
         * This section calculates net income attributable to BHC (i.e. BHCK4340)
		 ** Note: assume marginal tax rate = 35%.
         qui replace pretax_fc = cl_ppnr_fc - qllprov_fc + qafssecur_inc_fc + qhtmsecur_inc_fc in `j'
         qui replace qtaxes_inc_fc = ${taxrate} * pretax_fc in `j'
         
         * Note: assuming that minority interest stays fixed in dollar terms, and extraordinary items are zero
         qui replace qminor_int_fc = qminor_int_fc[`oldobs'] in `j'
         qui replace qextra_inc_fc = 0 in `j'
         qui replace qnetinc_fc = pretax_fc - qtaxes_inc_fc + qminor_int_fc + qextra_inc_fc in `j'
         
		 * Forecast dividends
         qui replace cl_dividend_fc = ${divar} * l1.cl_dividend_fc + (1-${divar}) * (${payout} * qnetinc_fc) in `j'
         qui replace cl_dividend_fc = 0 if cl_dividend_fc < 0 in `j'
			
         * Calculate total equity capital (assume constant dividends, no sale of common, preferred or Treasury stock, or other adjustments)
         qui replace equity_fc = l1.equity_fc + qnetinc_fc - cl_dividend_fc in `j'
         
         * Calculate regulatory capital subtotal (assuming wedge between this and total equity capital stays fixed over forecast horizon)
         qui replace t1_regcap_subtot_fc = l1.t1_regcap_subtot_fc + equity_fc - l1.equity_fc in `j'
         
         * Calculate total deferred tax balance
         qui replace dta_fc = max(l1.dta_fc - ${taxrate} * pretax_fc, 0) in `j'
         qui replace dta_allow_past_fc = dta_allow_past[`oldobs'] in `j'
         qui replace dta_total_fut_fc = dta_fc - dta_allow_past_fc in `j'
         qui replace dta_allow_fut_fc = l1.dta_allow_fut_fc + dta_fc - l1.dta_fc in `j'
         qui replace dta_allow_fut_fc = min(dta_allow_fut_fc , 0.1 * t1_regcap_subtot_fc) in `j'
         qui replace dta_dis_tot_fc = dta_fc - dta_allow_past_fc - dta_allow_fut_fc in `j'
         
         * Calculate tier 1 capital, risk-weighted assets and tier 1 capital ratio
         qui replace tier1_rbc_fc = l1.tier1_rbc_fc + (t1_regcap_subtot_fc - dta_dis_tot_fc) - (l1.t1_regcap_subtot_fc - l1.dta_dis_tot_fc) in `j'
         qui replace tier1_comm_fc = tier1_rbc_fc + tier1_comm[`oldobs'] - tier1_rbc[`oldobs'] in `j'
         qui replace rwa_fc = cl_assets * ( rwa[`oldobs'] / cl_assets[`oldobs'] ) in `j'
         qui replace cl_tier1_rat_fc = tier1_rbc_fc / rwa_fc * 100 in `j'
         qui replace cl_tier1_comm_rat_fc = tier1_comm_fc / rwa_fc * 100 in `j'
         qui replace liab_tot_fc = cl_assets - equity_fc in `j'	 
	}
		
      qui keep if _n >= `oldobs'
      qui replace name = name[`oldobs'] if name == ""
      
      * Drop the last four observations if used to calculate provisions
  	  if "${provisions}" == "four_smooth" {
         qui drop if _n > `oldobs' + `horizon' - 4
	  }

      if "${provisions}" == "four" {
         qui drop if _n > `oldobs' + `horizon' - 4
	  }
		
      if "${provisions}" == "tunnel" {
         qui drop if _n > `oldobs' + `horizon' - 4
	  }
		
	  if "${provisions}" == "tunnel_smooth" {
         qui drop if _n > `oldobs' + `horizon' - 4
	  }

      * Append the data to the overall forecast dataset across all BHCs
      if `firstbank' == 0 {
         qui save "output/tables/capital_forecasts_`scenario'_point.dta", replace
	  }
      
      if `firstbank' == 1 {
         qui append using "output/tables/capital_forecasts_`scenario'_point.dta"
         qui save "output/tables/capital_forecasts_`scenario'_point.dta", replace
	  }
      
      qui local firstbank = 1
      restore
	}
}

exit, clear

